@@ -18,17 +18,21 @@
 # http://www.gnu.org/copyleft/gpl.html
 
 /**
- *
- * @package MediaWiki
- * @subpackage SpecialPage
+ * @defgroup Dump Dump
  */
 
+/**
+ * @ingroup SpecialPage Dump
+ */
 class WikiExporter {
 	var $list_authors = false ; # Return distinct author list (when not returning full history)
 	var $author_list = "" ;
-	
+
+	var $dumpUploads = false;
+
 	const FULL = 0;
 	const CURRENT = 1;
+	const LOGS = 2;
 
 	const BUFFER = 0;
 	const STREAM = 1;
@@ -43,15 +47,15 @@ class WikiExporter {
 	 * make additional queries to pull source data while the
 	 * main query is still running.
 	 *
-	 * @param Database $db
-	 * @param mixed $history one of WikiExporter::FULL or WikiExporter::CURRENT, or an 
-	 *                       associative array:
-	 *                         offset: non-inclusive offset at which to start the query
-	 *                         limit: maximum number of rows to return
-	 *                         dir: "asc" or "desc" timestamp order
-	 * @param int $buffer one of WikiExporter::BUFFER or WikiExporter::STREAM
+	 * @param $db Database
+	 * @param $history Mixed: one of WikiExporter::FULL or WikiExporter::CURRENT,
+	 *                 or an associative array:
+	 *                   offset: non-inclusive offset at which to start the query
+	 *                   limit: maximum number of rows to return
+	 *                   dir: "asc" or "desc" timestamp order
+	 * @param $buffer Int: one of WikiExporter::BUFFER or WikiExporter::STREAM
 	 */
-	function WikiExporter( &$db, $history = WikiExporter::CURRENT,
+	function __construct( &$db, $history = WikiExporter::CURRENT,
 			$buffer = WikiExporter::BUFFER, $text = WikiExporter::TEXT ) {
 		$this->db =& $db;
 		$this->history = $history;
@@ -66,18 +70,18 @@ class WikiExporter {
 	 * various row objects and XML output for filtering. Filters
 	 * can be chained or used as callbacks.
 	 *
-	 * @param mixed $callback
+	 * @param $sink mixed
 	 */
-	function setOutputSink( &$sink ) {
+	public function setOutputSink( &$sink ) {
 		$this->sink =& $sink;
 	}
 
-	function openStream() {
+	public function openStream() {
 		$output = $this->writer->openStream();
 		$this->sink->writeOpenStream( $output );
 	}
 
-	function closeStream() {
+	public function closeStream() {
 		$output = $this->writer->closeStream();
 		$this->sink->writeCloseStream( $output );
 	}
@@ -87,18 +91,18 @@ class WikiExporter {
 	 * in the database, either including complete history or only
 	 * the most recent version.
 	 */
-	function allPages() {
+	public function allPages() {
 		return $this->dumpFrom( '' );
 	}
 
 	/**
 	 * Dumps a series of page and revision records for those pages
 	 * in the database falling within the page_id range given.
-	 * @param int $start Inclusive lower limit (this id is included)
-	 * @param int $end   Exclusive upper limit (this id is not included)
+	 * @param $start Int: inclusive lower limit (this id is included)
+	 * @param $end   Int: Exclusive upper limit (this id is not included)
 	 *                   If 0, no upper limit.
 	 */
-	function pagesByRange( $start, $end ) {
+	public function pagesByRange( $start, $end ) {
 		$condition = 'page_id >= ' . intval( $start );
 		if( $end ) {
 			$condition .= ' AND page_id < ' . intval( $end );
@@ -107,15 +111,15 @@ class WikiExporter {
 	}
 
 	/**
-	 * @param Title $title
+	 * @param $title Title
 	 */
-	function pageByTitle( $title ) {
+	public function pageByTitle( $title ) {
 		return $this->dumpFrom(
 			'page_namespace=' . $title->getNamespace() .
-			' AND page_title=' . $this->db->addQuotes( $title->getDbKey() ) );
+			' AND page_title=' . $this->db->addQuotes( $title->getDBkey() ) );
 	}
 
-	function pageByName( $name ) {
+	public function pageByName( $name ) {
 		$title = Title::newFromText( $name );
 		if( is_null( $title ) ) {
 			return new WikiError( "Can't export invalid title" );
@@ -124,120 +128,147 @@ class WikiExporter {
 		}
 	}
 
-	function pagesByName( $names ) {
+	public function pagesByName( $names ) {
 		foreach( $names as $name ) {
 			$this->pageByName( $name );
 		}
 	}
 
+	public function allLogs() {
+		return $this->dumpFrom( '' );
+	}
 
-	// -------------------- private implementation below --------------------
+	public function logsByRange( $start, $end ) {
+		$condition = 'log_id >= ' . intval( $start );
+		if( $end ) {
+			$condition .= ' AND log_id < ' . intval( $end );
+		}
+		return $this->dumpFrom( $condition );
+	}
 
 	# Generates the distinct list of authors of an article
 	# Not called by default (depends on $this->list_authors)
 	# Can be set by Special:Export when not exporting whole history
-	function do_list_authors ( $page , $revision , $cond ) {
+	protected function do_list_authors( $page , $revision , $cond ) {
 		$fname = "do_list_authors" ;
 		wfProfileIn( $fname );
 		$this->author_list = "<contributors>";
-		$sql = "SELECT DISTINCT rev_user_text,rev_user FROM {$page},{$revision} WHERE page_id=rev_page AND " . $cond ;
+		//rev_deleted
+		$nothidden = '(rev_deleted & '.Revision::DELETED_USER.') = 0';
+
+		$sql = "SELECT DISTINCT rev_user_text,rev_user FROM {$page},{$revision} 
+		WHERE page_id=rev_page AND $nothidden AND " . $cond ;
 		$result = $this->db->query( $sql, $fname );
 		$resultset = $this->db->resultObject( $result );
 		while( $row = $resultset->fetchObject() ) {
-			$this->author_list .= "<contributor>" . 
-				"<username>" . 
-				htmlentities( $row->rev_user_text )  . 
-				"</username>" . 
-				"<id>" . 
+			$this->author_list .= "<contributor>" .
+				"<username>" .
+				htmlentities( $row->rev_user_text )  .
+				"</username>" .
+				"<id>" .
 				$row->rev_user .
-				"</id>" . 
+				"</id>" .
 				"</contributor>";
 		}
 		wfProfileOut( $fname );
 		$this->author_list .= "</contributors>";
 	}
 
-	function dumpFrom( $cond = '' ) {
+	protected function dumpFrom( $cond = '' ) {
 		$fname = 'WikiExporter::dumpFrom';
 		wfProfileIn( $fname );
+		
+		# For logs dumps...
+		if( $this->history & self::LOGS ) {
+			$where = array( 'user_id = log_user' );
+			# Hide private logs
+			$where[] = LogEventsList::getExcludeClause( $this->db );
+			if( $cond ) $where[] = $cond;
+			$result = $this->db->select( array('logging','user'), 
+				'*',
+				$where,
+				$fname,
+				array( 'ORDER BY' => 'log_id', 'USE INDEX' => array('logging' => 'PRIMARY') )
+			);
+			$wrapper = $this->db->resultObject( $result );
+			$this->outputLogStream( $wrapper );
+		# For page dumps...
+		} else {
+			list($page,$revision,$text) = $this->db->tableNamesN('page','revision','text');
 
-		$page     = $this->db->tableName( 'page' );
-		$revision = $this->db->tableName( 'revision' );
-		$text     = $this->db->tableName( 'text' );
-		
-		$order = 'ORDER BY page_id';
-		$limit = '';
-		
-		if( $this->history == WikiExporter::FULL ) {
-			$join = 'page_id=rev_page';
-		} elseif( $this->history == WikiExporter::CURRENT ) {
-			if ( $this->list_authors && $cond != '' )  { // List authors, if so desired
-				$this->do_list_authors ( $page , $revision , $cond );
-			}
-			$join = 'page_id=rev_page AND page_latest=rev_id';
-		} elseif ( is_array( $this->history ) ) {
-			$join = 'page_id=rev_page';
-			if ( $this->history['dir'] == 'asc' ) {
-				$op = '>';
-				$order .= ', rev_timestamp';
-			} else {
-				$op = '<';
-				$order .= ', rev_timestamp DESC';
-			}
-			if ( !empty( $this->history['offset'] ) ) {
-				$join .= " AND rev_timestamp $op " . $this->db->addQuotes( 
-					$this->db->timestamp( $this->history['offset'] ) );
-			}
-			if ( !empty( $this->history['limit'] ) ) {
-				$limitNum = intval( $this->history['limit'] );
-				if ( $limitNum > 0 ) {
-					$limit = "LIMIT $limitNum";
+			$order = 'ORDER BY page_id';
+			$limit = '';
+
+			if( $this->history == WikiExporter::FULL ) {
+				$join = 'page_id=rev_page';
+			} elseif( $this->history == WikiExporter::CURRENT ) {
+				if ( $this->list_authors && $cond != '' )  { // List authors, if so desired
+					$this->do_list_authors ( $page , $revision , $cond );
 				}
+				$join = 'page_id=rev_page AND page_latest=rev_id';
+			} elseif ( is_array( $this->history ) ) {
+				$join = 'page_id=rev_page';
+				if ( $this->history['dir'] == 'asc' ) {
+					$op = '>';
+					$order .= ', rev_timestamp';
+				} else {
+					$op = '<';
+					$order .= ', rev_timestamp DESC';
+				}
+				if ( !empty( $this->history['offset'] ) ) {
+					$join .= " AND rev_timestamp $op " . $this->db->addQuotes(
+						$this->db->timestamp( $this->history['offset'] ) );
+				}
+				if ( !empty( $this->history['limit'] ) ) {
+					$limitNum = intval( $this->history['limit'] );
+					if ( $limitNum > 0 ) {
+						$limit = "LIMIT $limitNum";
+					}
+				}
+			} else {
+				wfProfileOut( $fname );
+				return new WikiError( "$fname given invalid history dump type." );
 			}
-		} else {
-			wfProfileOut( $fname );
-			return new WikiError( "$fname given invalid history dump type." );
-		}
-		$where = ( $cond == '' ) ? '' : "$cond AND";
+			$where = ( $cond == '' ) ? '' : "$cond AND";
 
-		if( $this->buffer == WikiExporter::STREAM ) {
-			$prev = $this->db->bufferResults( false );
-		}
-		if( $cond == '' ) {
-			// Optimization hack for full-database dump
-			$revindex = $pageindex = $this->db->useIndexClause("PRIMARY");
-			$straight = ' /*! STRAIGHT_JOIN */ ';
-		} else {
-			$pageindex = '';
-			$revindex = '';
-			$straight = '';
-		}
-		if( $this->text == WikiExporter::STUB ) {
-			$sql = "SELECT $straight * FROM
+			if( $this->buffer == WikiExporter::STREAM ) {
+				$prev = $this->db->bufferResults( false );
+			}
+			if( $cond == '' ) {
+				// Optimization hack for full-database dump
+				$revindex = $pageindex = $this->db->useIndexClause("PRIMARY");
+				$straight = ' /*! STRAIGHT_JOIN */ ';
+			} else {
+				$pageindex = '';
+				$revindex = '';
+				$straight = '';
+			}
+			if( $this->text == WikiExporter::STUB ) {
+				$sql = "SELECT $straight * FROM
 					$page $pageindex,
 					$revision $revindex
 					WHERE $where $join
 					$order $limit";
-		} else {
-			$sql = "SELECT $straight * FROM
+			} else {
+				$sql = "SELECT $straight * FROM
 					$page $pageindex,
 					$revision $revindex,
 					$text
 					WHERE $where $join AND rev_text_id=old_id
 					$order $limit";
-		}
-		$result = $this->db->query( $sql, $fname );
-		$wrapper = $this->db->resultObject( $result );
-		$this->outputStream( $wrapper );
-		
-		if ( $this->list_authors ) {
-			$this->outputStream( $wrapper );
-		}
+			}
+			$result = $this->db->query( $sql, $fname );
+			$wrapper = $this->db->resultObject( $result );
+			$this->outputPageStream( $wrapper );
 
-		if( $this->buffer == WikiExporter::STREAM ) {
-			$this->db->bufferResults( $prev );
-		}
+			if ( $this->list_authors ) {
+				$this->outputPageStream( $wrapper );
+			}
 
+			if( $this->buffer == WikiExporter::STREAM ) {
+				$this->db->bufferResults( $prev );
+			}
+		}
 		wfProfileOut( $fname );
 	}
 
@@ -251,17 +282,20 @@ class WikiExporter {
 	 * separate database connection not managed by LoadBalancer; some
 	 * blob storage types will make queries to pull source data.
 	 *
-	 * @param ResultWrapper $resultset
-	 * @access private
+	 * @param $resultset ResultWrapper
 	 */
-	function outputStream( $resultset ) {
+	protected function outputPageStream( $resultset ) {
 		$last = null;
 		while( $row = $resultset->fetchObject() ) {
 			if( is_null( $last ) ||
 				$last->page_namespace != $row->page_namespace ||
 				$last->page_title     != $row->page_title ) {
 				if( isset( $last ) ) {
-					$output = $this->writer->closePage();
+					$output = '';
+					if( $this->dumpUploads ) {
+						$output .= $this->writer->writeUploads( $last );
+					}
+					$output .= $this->writer->closePage();
 					$this->sink->writeClosePage( $output );
 				}
 				$output = $this->writer->openPage( $row );
@@ -272,13 +306,29 @@ class WikiExporter {
 			$this->sink->writeRevision( $row, $output );
 		}
 		if( isset( $last ) ) {
-			$output = $this->author_list . $this->writer->closePage();
+			$output = '';
+			if( $this->dumpUploads ) {
+				$output .= $this->writer->writeUploads( $last );
+			}
+			$output .= $this->author_list;
+			$output .= $this->writer->closePage();
 			$this->sink->writeClosePage( $output );
 		}
 		$resultset->free();
 	}
+	
+	protected function outputLogStream( $resultset ) {
+		while( $row = $resultset->fetchObject() ) {
+			$output = $this->writer->writeLogItem( $row );
+			$this->sink->writeLogItem( $row, $output );
+		}
+		$resultset->free();
+	}
 }
 
+/**
+ * @ingroup Dump
+ */
 class XmlDumpWriter {
 
 	/**
@@ -302,7 +352,7 @@ class XmlDumpWriter {
 	function openStream() {
 		global $wgContLanguageCode;
 		$ver = $this->schemaVersion();
-		return wfElement( 'mediawiki', array(
+		return Xml::element( 'mediawiki', array(
 			'xmlns'              => "http://www.mediawiki.org/xml/export-$ver/",
 			'xmlns:xsi'          => "http://www.w3.org/2001/XMLSchema-instance",
 			'xsi:schemaLocation' => "http://www.mediawiki.org/xml/export-$ver/ " .
@@ -328,30 +378,30 @@ class XmlDumpWriter {
 
 	function sitename() {
 		global $wgSitename;
-		return wfElement( 'sitename', array(), $wgSitename );
+		return Xml::element( 'sitename', array(), $wgSitename );
 	}
 
 	function generator() {
 		global $wgVersion;
-		return wfElement( 'generator', array(), "MediaWiki $wgVersion" );
+		return Xml::element( 'generator', array(), "MediaWiki $wgVersion" );
 	}
 
 	function homelink() {
-		return wfElement( 'base', array(), Title::newMainPage()->getFullUrl() );
+		return Xml::element( 'base', array(), Title::newMainPage()->getFullUrl() );
 	}
 
 	function caseSetting() {
 		global $wgCapitalLinks;
 		// "case-insensitive" option is reserved for future
 		$sensitivity = $wgCapitalLinks ? 'first-letter' : 'case-sensitive';
-		return wfElement( 'case', array(), $sensitivity );
+		return Xml::element( 'case', array(), $sensitivity );
 	}
 
 	function namespaces() {
 		global $wgContLang;
 		$spaces = "  <namespaces>\n";
 		foreach( $wgContLang->getFormattedNamespaces() as $ns => $title ) {
-			$spaces .= '      ' . wfElement( 'namespace', array( 'key' => $ns ), $title ) . "\n";
+			$spaces .= '      ' . Xml::element( 'namespace', array( 'key' => $ns ), $title ) . "\n";
 		}
 		$spaces .= "    </namespaces>";
 		return $spaces;
@@ -370,17 +420,17 @@ class XmlDumpWriter {
 	 * Opens a <page> section on the output stream, with data
 	 * from the given database row.
 	 *
-	 * @param object $row
+	 * @param $row object
 	 * @return string
 	 * @access private
 	 */
 	function openPage( $row ) {
 		$out = "  <page>\n";
 		$title = Title::makeTitle( $row->page_namespace, $row->page_title );
-		$out .= '    ' . wfElementClean( 'title', array(), $title->getPrefixedText() ) . "\n";
-		$out .= '    ' . wfElement( 'id', array(), strval( $row->page_id ) ) . "\n";
+		$out .= '    ' . Xml::elementClean( 'title', array(), $title->getPrefixedText() ) . "\n";
+		$out .= '    ' . Xml::element( 'id', array(), strval( $row->page_id ) ) . "\n";
 		if( '' != $row->page_restrictions ) {
-			$out .= '    ' . wfElement( 'restrictions', array(),
+			$out .= '    ' . Xml::element( 'restrictions', array(),
 				strval( $row->page_restrictions ) ) . "\n";
 		}
 		return $out;
@@ -399,7 +449,7 @@ class XmlDumpWriter {
 	 * Dumps a <revision> section on the output stream, with
 	 * data filled in from the given database row.
 	 *
-	 * @param object $row
+	 * @param $row object
 	 * @return string
 	 * @access private
 	 */
@@ -408,44 +458,36 @@ class XmlDumpWriter {
 		wfProfileIn( $fname );
 
 		$out  = "    <revision>\n";
-		$out .= "      " . wfElement( 'id', null, strval( $row->rev_id ) ) . "\n";
+		$out .= "      " . Xml::element( 'id', null, strval( $row->rev_id ) ) . "\n";
 
-		$ts = wfTimestamp( TS_ISO_8601, $row->rev_timestamp );
-		$out .= "      " . wfElement( 'timestamp', null, $ts ) . "\n";
+		$out .= $this->writeTimestamp( $row->rev_timestamp );
 
 		if( $row->rev_deleted & Revision::DELETED_USER ) {
-			$out .= "      " . wfElement( 'contributor', array( 'deleted' => 'deleted' ) ) . "\n";
+			$out .= "      " . Xml::element( 'contributor', array( 'deleted' => 'deleted' ) ) . "\n";
 		} else {
-			$out .= "      <contributor>\n";
-			if( $row->rev_user ) {
-				$out .= "        " . wfElementClean( 'username', null, strval( $row->rev_user_text ) ) . "\n";
-				$out .= "        " . wfElement( 'id', null, strval( $row->rev_user ) ) . "\n";
-			} else {
-				$out .= "        " . wfElementClean( 'ip', null, strval( $row->rev_user_text ) ) . "\n";
-			}
-			$out .= "      </contributor>\n";
+			$out .= $this->writeContributor( $row->rev_user, $row->rev_user_text );
 		}
 
 		if( $row->rev_minor_edit ) {
 			$out .=  "      <minor/>\n";
 		}
 		if( $row->rev_deleted & Revision::DELETED_COMMENT ) {
-			$out .= "      " . wfElement( 'comment', array( 'deleted' => 'deleted' ) ) . "\n";
+			$out .= "      " . Xml::element( 'comment', array( 'deleted' => 'deleted' ) ) . "\n";
 		} elseif( $row->rev_comment != '' ) {
-			$out .= "      " . wfElementClean( 'comment', null, strval( $row->rev_comment ) ) . "\n";
+			$out .= "      " . Xml::elementClean( 'comment', null, strval( $row->rev_comment ) ) . "\n";
 		}
 
 		if( $row->rev_deleted & Revision::DELETED_TEXT ) {
-			$out .= "      " . wfElement( 'text', array( 'deleted' => 'deleted' ) ) . "\n";
+			$out .= "      " . Xml::element( 'text', array( 'deleted' => 'deleted' ) ) . "\n";
 		} elseif( isset( $row->old_text ) ) {
 			// Raw text from the database may have invalid chars
 			$text = strval( Revision::getRevisionText( $row ) );
-			$out .= "      " . wfElementClean( 'text',
+			$out .= "      " . Xml::elementClean( 'text',
 				array( 'xml:space' => 'preserve' ),
 				strval( $text ) ) . "\n";
 		} else {
 			// Stub output
-			$out .= "      " . wfElement( 'text',
+			$out .= "      " . Xml::element( 'text',
 				array( 'id' => $row->rev_text_id ),
 				"" ) . "\n";
 		}
@@ -455,12 +497,107 @@ class XmlDumpWriter {
 		wfProfileOut( $fname );
 		return $out;
 	}
+	
+	/**
+	 * Dumps a <logitem> section on the output stream, with
+	 * data filled in from the given database row.
+	 *
+	 * @param $row object
+	 * @return string
+	 * @access private
+	 */
+	function writeLogItem( $row ) {
+		$fname = 'WikiExporter::writeLogItem';
+		wfProfileIn( $fname );
+
+		$out  = "    <logitem>\n";
+		$out .= "      " . Xml::element( 'id', null, strval( $row->log_id ) ) . "\n";
+
+		$out .= $this->writeTimestamp( $row->log_timestamp );
+
+		if( $row->log_deleted & LogPage::DELETED_USER ) {
+			$out .= "      " . Xml::element( 'contributor', array( 'deleted' => 'deleted' ) ) . "\n";
+		} else {
+			$out .= $this->writeContributor( $row->log_user, $row->user_name );
+		}
+
+		if( $row->log_deleted & LogPage::DELETED_COMMENT ) {
+			$out .= "      " . Xml::element( 'comment', array( 'deleted' => 'deleted' ) ) . "\n";
+		} elseif( $row->log_comment != '' ) {
+			$out .= "      " . Xml::elementClean( 'comment', null, strval( $row->log_comment ) ) . "\n";
+		}
+		
+		$out .= "      " . Xml::element( 'type', null, strval( $row->log_type ) ) . "\n";
+		$out .= "      " . Xml::element( 'action', null, strval( $row->log_action ) ) . "\n";
+
+		if( $row->log_deleted & LogPage::DELETED_ACTION ) {
+			$out .= "      " . Xml::element( 'text', array( 'deleted' => 'deleted' ) ) . "\n";
+		} else {
+			$title = Title::makeTitle( $row->log_namespace, $row->log_title );
+			$out .= "      " . Xml::elementClean( 'logtitle', null, $title->getPrefixedText() ) . "\n";
+			$out .= "      " . Xml::elementClean( 'params',
+				array( 'xml:space' => 'preserve' ),
+				strval( $row->log_params ) ) . "\n";
+		}
+
+		$out .= "    </logitem>\n";
+
+		wfProfileOut( $fname );
+		return $out;
+	}
+
+	function writeTimestamp( $timestamp ) {
+		$ts = wfTimestamp( TS_ISO_8601, $timestamp );
+		return "      " . Xml::element( 'timestamp', null, $ts ) . "\n";
+	}
+
+	function writeContributor( $id, $text ) {
+		$out = "      <contributor>\n";
+		if( $id ) {
+			$out .= "        " . Xml::elementClean( 'username', null, strval( $text ) ) . "\n";
+			$out .= "        " . Xml::element( 'id', null, strval( $id ) ) . "\n";
+		} else {
+			$out .= "        " . Xml::elementClean( 'ip', null, strval( $text ) ) . "\n";
+		}
+		$out .= "      </contributor>\n";
+		return $out;
+	}
+
+	/**
+	 * Warning! This data is potentially inconsistent. :(
+	 */
+	function writeUploads( $row ) {
+		if( $row->page_namespace == NS_IMAGE ) {
+			$img = wfFindFile( $row->page_title );
+			if( $img ) {
+				$out = '';
+				foreach( array_reverse( $img->getHistory() ) as $ver ) {
+					$out .= $this->writeUpload( $ver );
+				}
+				$out .= $this->writeUpload( $img );
+				return $out;
+			}
+		}
+		return '';
+	}
+
+	function writeUpload( $file ) {
+		return "    <upload>\n" .
+			$this->writeTimestamp( $file->getTimestamp() ) .
+			$this->writeContributor( $file->getUser( 'id' ), $file->getUser( 'text' ) ) .
+			"      " . Xml::elementClean( 'comment', null, $file->getDescription() ) . "\n" .
+			"      " . Xml::element( 'filename', null, $file->getName() ) . "\n" .
+			"      " . Xml::element( 'src', null, $file->getFullUrl() ) . "\n" .
+			"      " . Xml::element( 'size', null, $file->getSize() ) . "\n" .
+			"    </upload>\n";
+	}
 
 }
 
 
 /**
  * Base class for output stream; prints to stdout or buffer or whereever.
+ * @ingroup Dump
  */
 class DumpOutput {
 	function writeOpenStream( $string ) {
@@ -482,6 +619,10 @@ class DumpOutput {
 	function writeRevision( $rev, $string ) {
 		$this->write( $string );
 	}
+	
+	function writeLogItem( $rev, $string ) {
+		$this->write( $string );
+	}
 
 	/**
 	 * Override to write to a different stream type.
@@ -494,6 +635,7 @@ class DumpOutput {
 
 /**
  * Stream outputter to send data to a file.
+ * @ingroup Dump
  */
 class DumpFileOutput extends DumpOutput {
 	var $handle;
@@ -511,6 +653,7 @@ class DumpFileOutput extends DumpOutput {
  * Stream outputter to send data to a file via some filter program.
  * Even if compression is available in a library, using a separate
  * program can allow us to make use of a multi-processor system.
+ * @ingroup Dump
  */
 class DumpPipeOutput extends DumpFileOutput {
 	function DumpPipeOutput( $command, $file = null ) {
@@ -523,6 +666,7 @@ class DumpPipeOutput extends DumpFileOutput {
 
 /**
  * Sends dump output via the gzip compressor.
+ * @ingroup Dump
  */
 class DumpGZipOutput extends DumpPipeOutput {
 	function DumpGZipOutput( $file ) {
@@ -532,6 +676,7 @@ class DumpGZipOutput extends DumpPipeOutput {
 
 /**
  * Sends dump output via the bgzip2 compressor.
+ * @ingroup Dump
  */
 class DumpBZip2Output extends DumpPipeOutput {
 	function DumpBZip2Output( $file ) {
@@ -541,13 +686,14 @@ class DumpBZip2Output extends DumpPipeOutput {
 
 /**
  * Sends dump output via the p7zip compressor.
+ * @ingroup Dump
  */
 class Dump7ZipOutput extends DumpPipeOutput {
 	function Dump7ZipOutput( $file ) {
 		$command = "7za a -bd -si " . wfEscapeShellArg( $file );
 		// Suppress annoying useless crap from p7zip
 		// Unfortunately this could suppress real error messages too
-		$command .= " >/dev/null 2>&1";
+		$command .= ' >' . wfGetNull() . ' 2>&1';
 		parent::DumpPipeOutput( $command );
 	}
 }
@@ -558,6 +704,7 @@ class Dump7ZipOutput extends DumpPipeOutput {
  * Dump output filter class.
  * This just does output filtering and streaming; XML formatting is done
  * higher up, so be careful in what you do.
+ * @ingroup Dump
  */
 class DumpFilter {
 	function DumpFilter( &$sink ) {
@@ -591,6 +738,10 @@ class DumpFilter {
 			$this->sink->writeRevision( $rev, $string );
 		}
 	}
+	
+	function writeLogItem( $rev, $string ) {
+		$this->sink->writeRevision( $rev, $string );
+	}	
 
 	/**
 	 * Override for page-based filter types.
@@ -603,15 +754,17 @@ class DumpFilter {
 
 /**
  * Simple dump output filter to exclude all talk pages.
+ * @ingroup Dump
  */
 class DumpNotalkFilter extends DumpFilter {
 	function pass( $page ) {
-		return !Namespace::isTalk( $page->page_namespace );
+		return !MWNamespace::isTalk( $page->page_namespace );
 	}
 }
 
 /**
  * Dump output filter to include or exclude pages in a given set of namespaces.
+ * @ingroup Dump
  */
 class DumpNamespaceFilter extends DumpFilter {
 	var $invert = false;
@@ -627,7 +780,9 @@ class DumpNamespaceFilter extends DumpFilter {
 			"NS_USER_TALK"      => NS_USER_TALK,
 			"NS_PROJECT"        => NS_PROJECT,
 			"NS_PROJECT_TALK"   => NS_PROJECT_TALK,
-			"NS_IMAGE"          => NS_IMAGE,
+			"NS_FILE"           => NS_FILE,
+			"NS_FILE_TALK"      => NS_FILE_TALK,
+			"NS_IMAGE"          => NS_IMAGE,  // NS_IMAGE is an alias for NS_FILE
 			"NS_IMAGE_TALK"     => NS_IMAGE_TALK,
 			"NS_MEDIAWIKI"      => NS_MEDIAWIKI,
 			"NS_MEDIAWIKI_TALK" => NS_MEDIAWIKI_TALK,
@@ -666,6 +821,7 @@ class DumpNamespaceFilter extends DumpFilter {
 
 /**
  * Dump output filter to include only the last revision in each page sequence.
+ * @ingroup Dump
  */
 class DumpLatestFilter extends DumpFilter {
 	var $page, $pageString, $rev, $revString;
@@ -697,6 +853,7 @@ class DumpLatestFilter extends DumpFilter {
 
 /**
  * Base class for output stream; prints to stdout or buffer or whereever.
+ * @ingroup Dump
  */
 class DumpMultiWriter {
 	function DumpMultiWriter( $sinks ) {
@@ -750,5 +907,3 @@ function xmlsafe( $string ) {
 	wfProfileOut( $fname );
 	return $string;
 }
-
-?>
