@@ -1,7 +1,10 @@
 <?php
 /**
  * File for magic words
- * @addtogroup Parser
+ * See docs/magicword.txt
+ *
+ * @file
+ * @ingroup Parser
  */
 
 /**
@@ -16,10 +19,11 @@
  * Please avoid reading the data out of one of these objects and then writing
  * special case code. If possible, add another match()-like function here.
  *
- * To add magic words in an extension, use the LanguageGetMagic hook. For 
+ * To add magic words in an extension, use the LanguageGetMagic hook. For
  * magic words which are also Parser variables, add a MagicWordwgVariableIDs
  * hook. Use string keys.
  *
+ * @ingroup Parser
  */
 class MagicWord {
 	/**#@+
@@ -99,10 +103,70 @@ class MagicWord {
 		'contentlanguage',
 		'pagesinnamespace',
 		'numberofadmins',
+		'numberofviews',
 		'defaultsort',
+		'pagesincategory',
+		'index',
+		'noindex',
+		'numberingroup',
 	);
 
+	/* Array of caching hints for ParserCache */
+	static public $mCacheTTLs = array (
+		'currentmonth' => 86400,
+		'currentmonthname' => 86400,
+		'currentmonthnamegen' => 86400,
+		'currentmonthabbrev' => 86400,
+		'currentday' => 3600,
+		'currentday2' => 3600,
+		'currentdayname' => 3600,
+		'currentyear' => 86400,
+		'currenttime' => 3600,
+		'currenthour' => 3600,
+		'localmonth' => 86400,
+		'localmonthname' => 86400,
+		'localmonthnamegen' => 86400,
+		'localmonthabbrev' => 86400,
+		'localday' => 3600,
+		'localday2' => 3600,
+		'localdayname' => 3600,
+		'localyear' => 86400,
+		'localtime' => 3600,
+		'localhour' => 3600,
+		'numberofarticles' => 3600,
+		'numberoffiles' => 3600,
+		'numberofedits' => 3600,
+		'currentweek' => 3600,
+		'currentdow' => 3600,
+		'localweek' => 3600,
+		'localdow' => 3600,
+		'numberofusers' => 3600,
+		'numberofpages' => 3600,
+		'currentversion' => 86400,
+		'currenttimestamp' => 3600,
+		'localtimestamp' => 3600,
+		'pagesinnamespace' => 3600,
+		'numberofadmins' => 3600,
+		'numberofviews' => 3600,
+		'numberingroup' => 3600,
+		);
+
+	static public $mDoubleUnderscoreIDs = array(
+		'notoc',
+		'nogallery',
+		'forcetoc',
+		'toc',
+		'noeditsection',
+		'newsectionlink',
+		'hiddencat',
+		'index',
+		'noindex',
+		'staticredirect',
+	);
+
+
 	static public $mObjects = array();
+	static public $mDoubleUnderscoreArray = null;
 
 	/**#@-*/
 
@@ -122,11 +186,13 @@ class MagicWord {
 	 * @static
 	 */
 	static function &get( $id ) {
+		wfProfileIn( __METHOD__ );
 		if (!array_key_exists( $id, self::$mObjects ) ) {
 			$mw = new MagicWord();
 			$mw->load( $id );
 			self::$mObjects[$id] = $mw;
 		}
+		wfProfileOut( __METHOD__ );
 		return self::$mObjects[$id];
 	}
 
@@ -149,6 +215,23 @@ class MagicWord {
 		return self::$mVariableIDs;
 	}
 
+	/* Allow external reads of TTL array */
+	static function getCacheTTL($id) {
+		if (array_key_exists($id,self::$mCacheTTLs)) {
+			return self::$mCacheTTLs[$id];
+		} else {
+			return -1;
+		}
+	}
+
+	/** Get a MagicWordArray of double-underscore entities */
+	static function getDoubleUnderscoreArray() {
+		if ( is_null( self::$mDoubleUnderscoreArray ) ) {
+			self::$mDoubleUnderscoreArray = new MagicWordArray( self::$mDoubleUnderscoreIDs );
+		}
+		return self::$mDoubleUnderscoreArray;
+	}
+
 	# Initialises this object with an ID
 	function load( $id ) {
 		global $wgContLang;
@@ -170,13 +253,13 @@ class MagicWord {
 		# This was used for matching "$1" variables, but different uses of the feature will have
 		# different restrictions, which should be checked *after* the MagicWord has been matched,
 		# not here. - IMSoP
-		
+
 		$escSyn = array();
 		foreach ( $this->mSynonyms as $synonym )
 			// In case a magic word contains /, like that's going to happen;)
 			$escSyn[] = preg_quote( $synonym, '/' );
 		$this->mBaseRegex = implode( '|', $escSyn );
-		
+
 		$case = $this->mCaseSensitive ? '' : 'iu';
 		$this->mRegex = "/{$this->mBaseRegex}/{$case}";
 		$this->mRegexStart = "/^(?:{$this->mBaseRegex})/{$case}";
@@ -394,11 +477,13 @@ class MagicWord {
 
 /**
  * Class for handling an array of magic words
+ * @ingroup Parser
  */
 class MagicWordArray {
 	var $names = array();
 	var $hash;
 	var $baseRegex, $regex;
+	var $matches;
 
 	function __construct( $names = array() ) {
 		$this->names = $names;
@@ -505,6 +590,8 @@ class MagicWordArray {
 
 	/**
 	 * Parse a match array from preg_match
+	 * Returns array(magic word ID, parameter value)
+	 * If there is no parameter value, that element will be false.
 	 */
 	function parseMatch( $m ) {
 		reset( $m );
@@ -529,7 +616,7 @@ class MagicWordArray {
 
 	/**
 	 * Match some text, with parameter capture
-	 * Returns an array with the magic word name in the first element and the 
+	 * Returns an array with the magic word name in the first element and the
 	 * parameter in the second element.
 	 * Both elements are false if there was no match.
 	 */
@@ -563,4 +650,25 @@ class MagicWordArray {
 		}
 		return false;
 	}
+
+	/**
+	 * Returns an associative array, ID => param value, for all items that match
+	 * Removes the matched items from the input string (passed by reference)
+	 */
+	public function matchAndRemove( &$text ) {
+		$found = array();
+		$regexes = $this->getRegex();
+		foreach ( $regexes as $regex ) {
+			if ( $regex === '' ) {
+				continue;
+			}
+			preg_match_all( $regex, $text, $matches, PREG_SET_ORDER );
+			foreach ( $matches as $m ) {
+				list( $name, $param ) = $this->parseMatch( $m );
+				$found[$name] = $param;
+			}
+			$text = preg_replace( $regex, '', $text );
+		}
+		return $found;
+	}
 }
