@@ -1,7 +1,14 @@
 <?php
+/**
+ * Delete one or more revisions by moving them to the archive table.
+ *
+ * @file
+ * @ingroup Maintenance
+ */
+
 require_once( 'commandLine.inc' );
 
-$dbw =& wfGetDB( DB_MASTER );
+$dbw = wfGetDB( DB_MASTER );
 
 if ( count( $args ) == 0 ) {
 	echo "Usage: php deleteRevision.php <revid> [<revid> ...]\n";
@@ -37,4 +44,4 @@ foreach ( $args as $revID ) {
 }
 
 print "Deleted $affected revisions\n";
-?>
+
